#!/usr/bin/env python
# -*- coding: utf-8 -*-

STORAGE = 'gottwall.storages.RedisStorage'

BACKENDS = {
    'gottwall.backends.redis.RedisBackend': {
        'HOST': "127.0.0.1",
        'PORT': 6379,
        'PASSWORD': None,
        'DB': 2,
        "CHANNEL": "gottwall"},
    'gottwall.backends.tcpip.TCPIPBackend': {}
    }

TEMPLATE_DEBUG = True

STORAGE_SETTINGS = dict(
    HOST = 'localhost',
    PORT = 6379,
    PASSWORD = None,
    DB = 2
)

REDIS = {"CHANNEL": "gottwall"}


USERS = ["sergeevvv@gmail.com", "alexandr.s.rus@gmail.com"]

SECRET_KEY = "dwefwefwefwecwef"

# http://public_key:secret_key@host.com

PROJECTS = {"test_project": "my_public_key",
            "another_project": "public_key2"}

cookie_secret="fkewrlhfwhrfweiurbweuybfrweoubfrowebfioubweoiufbwbeofbowebfbwup2XdTP1o/Vo="

TEMPLATE_DEBUG = True


DATABASE = {
    "ENGINE": "postgresql+psycopg2",
    "HOST": "localhost",
    "PORT": 5432,
    "USER": "postgres",
    "PASSWORD": "none",
    "NAME": "gottwall"
    }
PREFIX = ""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import datetime
import unittest

from gottwall.utils import timestamp_to_datetime, get_by_period, MagicDict
import tornadoredis


def async_test_ex(timeout=5):
    def _inner(func):
        def _runner(self, *args, **kwargs):
            try:
                func(self, *args, **kwargs)
            except Exception, e:
                print(e)
                self.stop()
                raise
            return self.wait(timeout=timeout)
        return _runner
    return _inner


def async_test(func):
    _inner = async_test_ex()
    return _inner(func)


class UtilsTestCase(unittest.TestCase):
    def test_timestamp_to_datetime(self):
        d = datetime(2012, 11, 1, 3, 4, 5)
        timestamp = d.strftime("%Y-%m-%dT%H:%M:%S")
        self.assertEquals(d, timestamp_to_datetime(timestamp))


    def test_magic_dict(self):
        magic_dict = MagicDict()

        self.assertTrue(isinstance(magic_dict['key'], MagicDict))






#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.storages
~~~~~~~~~~~~~~~~~

GottWall storages backends

:copyright: (c) 2012 by GottWall Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/gottwall/gottwall
"""
from itertools import ifilter
from logging import getLogger

from gottwall.compat import OrderedDict
from gottwall.utils import get_by_period, date_range, date_min, date_max, get_datetime


logger = getLogger("gottwall.storages")


class BaseStorage(object):
    """Base interface for calculation
    """

    def __init__(self, application):
        self._application = application

    @classmethod
    def setup(cls, application):
        """Setup storage to application
        """
        storage = cls(application)
        application.storage = storage
        return storage


    def incr(self, project, name, timestamp, value=1, filters=None, **kwargs):
        """Add count for metric `name` and `filters`

        :param project: project name
        :param name: metric name
        :param filters: list of names
        :param value: increment value
        """
        raise NotImplementedError

    def decr(self, project, name, timestamp, value=-1, filters=None, **kwargs):
        """Sub value from metric `name` in `project`

        :param project: project name
        :param name: metric name
        :param filters: list of names
        :param value: increment value
        """
        raise NotImplementedError

    def slice_data(self, period, from_date, to_date, filter_name, filter_value):
        """Get data by range and filters

        :param period: range periodic
        :param from_date: slice from
        :param to_date: slice to
        :param filter_name: slice for filter
        :param filter_value: slice for filter value
        """
        raise NotImplementedError

    def metrics(self, project):
        """Get metrics
        """
        raise NotImplementedError

    def convert_range_to_datetime(self, data, period):
        return map(lambda x: (get_datetime(x[0], period), x[1]), data)

    def filter_by_period(self, data, period, from_date=None, to_date=None):
        """Fulter statistics by `from_date` and `to_date`

        :param from_data:
        :param to_date:
        :return: ifilter generator

        """
        from_date = date_min(from_date, period)
        to_date = date_max(to_date, period)

        data = self.convert_range_to_datetime(data, period)

        if from_date and to_date:
            new_data = OrderedDict(map(lambda x: (x, 0),
                                       date_range(from_date, to_date, period)))
            new_data.update(data)
        else:
            new_data = OrderedDict(data)

        # Convert datestring to datetime object in list


        new_data = sorted(ifilter(lambda x: (True if from_date is None else x[0] >= from_date) and \
                                  (True if to_date is None else x[0] <= to_date), new_data.items()),
                          key=lambda x: x[0])

        return map(lambda x: (get_by_period(x[0], period), x[1]), new_data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.config
~~~~~~~~~~~~~~~

Gottwall configs wrapper

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/gottwall/gottwall
"""
import imp
import os.path

from settings import PERIODS


class Config(dict):
    """Settings store
    """
    def __init__(self, **kwargs):
        dict.__init__(self, kwargs or {})

    def from_file(self, filename):
        """Load settings from pyfile with `filename`
        """
        filename = os.path.abspath(filename)
        d = imp.new_module('config')
        d.__file__ = filename
        try:
            execfile(filename, d.__dict__)
        except IOError, e:
            raise Exception("Unable to load configuration file {0}: ".\
                            format(filename, e))

        self.from_object(d)

        return True

    def from_object(self, obj):
        """Load settings from obj

        :param obj: object instance
        """
        for key in dir(obj):
            if not key.startswith('__'):
                self[key] = getattr(obj, key)
        return True

    def from_module(self, m):
        """Load settings from module

        :param m: module instance
        """
        return self.from_object(m)

    def __repr__(self):
        return '<{0} {1}>'.format(self.__class__.__name__, dict.__repr__(self))

default_settings = Config(**{
    "BACKENDS": ['gottwall.backends.HTTPBackend'],
    "STORAGE": "gottwall.storages.MemoryStorage",
    "PERIODS": PERIODS,
    "PROJECTS": {},
    "USERS": []})



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.runner
~~~~~~~~~~~~~~~

GottWall runner for standalone applications

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details..
:license: BSD, see LICENSE for more details.
:github: http://github.com/gottwall/gottwall
"""
import os.path
import sys
import time

import signal
import logging as logging_module
from logging import StreamHandler
from optparse import OptionParser, Option

import tornado.ioloop
from commandor import Command, Commandor
from tornado import httpserver, autoreload
from tornado.options import _LogFormatter

import gottwall.default_config
from gottwall.aggregator import AggregatorApplication
from gottwall.app import HTTPApplication
from gottwall.config import Config
from gottwall.log import logger


def configure_logging(logging):
    """Configure logging handler"""
    if logging.upper() not in ['DEBUG', 'INFO', 'CRITICAL',
                               'WARNING', 'ERROR']:
        return

    logger.setLevel(getattr(logging_module, logging.upper()))

    if not logger.handlers:
        channel = StreamHandler()
        channel.setFormatter(_LogFormatter(color=False))
        logger.addHandler(channel)
    logger.info("Logging handler configured with level {0}".format(logging))


class Commandor(Commandor):
    """Arguments management utilities
    """

    def run(self, options, args):
        """Prepare env befor command execution

        :param options: options object
        :param args: list or commandor args
        """

        if not options.config:
            self.error("You need specify --config\n")
            super(Commandor, self).run(options, args)
            self.exit()

        config = Config()

        # Load default config
        config.from_module(gottwall.default_config)

        # Rewrite default settings
        config.from_file(options.config)

        return config


class Aggregator(Command):
    """Tools for aggregator
    """
    commandor = Commandor


class Start(Command):
    """Aggregator starter
    """
    parent = Aggregator

    options = [
        Option("-p", "--port",
               metavar=int,
               default=8890,
               help="Port to run http server"),
        Option("-r", "--reload",
               action="store_true",
               dest="reload",
               default=False,
               help="Auto realod source on changes"),
        Option("-h", "--host",
               metavar="str",
               default="127.0.0.1",
               help="Port for server"),
        Option("-l", "--logging",
               metavar="str",
               default="none",
               help="Log level")]

    def run(self, port, reload, host, logging, **kwargs):
        config = self._commandor_res

        configure_logging(logging)

        self.application = AggregatorApplication(config)
        ioloop = tornado.ioloop.IOLoop.instance()

        self.application.configure_app(ioloop)

        self.http_server = httpserver.HTTPServer(self.application)
        self.http_server.listen(str(port), host)

        if reload:
            self.display("Autoreload enabled")
            autoreload.start(io_loop=ioloop, check_time=100)

        self.display("Aggregator running on 127.0.0.1:{0}".format(port))

        # Init signals handler
        signal.signal(signal.SIGTERM, self.sig_handler)

        # This will also catch KeyboardInterrupt exception
        signal.signal(signal.SIGINT, self.sig_handler)

        ioloop.start()

    def sig_handler(self, sig, frame):
        """Catch signal and init callback
        """
        tornado.ioloop.IOLoop.instance().add_callback(self.shutdown)

    def shutdown(self):
        """Stop server and add callback to stop i/o loop"""
        self.display("Shutting down service")
        self.application.shutdown()
        self.http_server.stop()
        self.application.check_ready_to_stop(self.application)


class Server(Command):
    """Server commands
    """
    commandor = Commandor


class Start(Command):
    """Run server
    """
    parent = Server

    options = [
        Option("-p", "--port",
               metavar=int,
               default=8889,
               help="Port to run http server"),
        Option("-r", "--reload",
               action="store_true",
               dest="reload",
               default=False,
               help="Auto realod source on changes"),
        Option("-h", "--host",
               metavar="str",
               default="127.0.0.1",
               help="Port for server"),
        Option("-l", "--logging",
               metavar="str",
               default="none",
               help="Log level")]

    def run(self, port, reload, host, logging, **kwargs):
        config = self._commandor_res

        configure_logging(logging)

        application = HTTPApplication(config)
        ioloop = tornado.ioloop.IOLoop.instance()

        application.configure_app(ioloop)

        self.http_server = httpserver.HTTPServer(application)
        self.http_server.listen(port, host)

        if reload:
            self.display("Autoreload enabled")
            autoreload.start(io_loop=ioloop, check_time=100)

        self.display("Server running on 127.0.0.1:{0}".format(port))

        # Init signals handler
        signal.signal(signal.SIGTERM, self.sig_handler)

        # This will also catch KeyboardInterrupt exception
        signal.signal(signal.SIGINT, self.sig_handler)

        ioloop.start()

    def sig_handler(self, sig, frame):
        """Catch signal and init callback
        """
        tornado.ioloop.IOLoop.instance().add_callback(self.shutdown)

    def shutdown(self):
        """Stop server and add callback to stop i/o loop"""
        self.display("Shutting down service")

        self.http_server.stop()

        io_loop = tornado.ioloop.IOLoop.instance()
        io_loop.add_timeout(time.time() + 2, io_loop.stop)


class Alembic(Command):
    """SqlAlchemy migration tools
    """
    commandor = Commandor

    # ALBEMIC INTEGRATION
    def __init__(self, *args, **kwargs):
        from alembic.config import CommandLine

        kwargs['parser'] = CommandLine(self.__class__.__name__).parser
        super(Alembic, self).__init__(*args, **kwargs)

    def process(self):
        """Execute command
        """
        self.configure()

        options = self.parse_args()
        return self.run(options, *options.cmd)

    def run(self, options, fn, positional, kwarg):
        from alembic.config import Config as AlembicConfig
        from alembic import util

        config = self._commandor_res

        alembic_cfg = AlembicConfig(os.path.join(os.path.dirname(
            gottwall.default_config.__file__), 'alembic.ini'))
        alembic_cfg.set_main_option("script_location",
                                   config['ALEMBIC_SCRIPT_LOCATION'])

        alembic_cfg.set_main_option("sqlalchemy.url",
                                    "{ENGINE}://{USER}:{PASSWORD}@{HOST}:{PORT}/{NAME}".\
                                    format(**config['DATABASE']))

        try:
            fn(alembic_cfg,
               *[getattr(options, k) for k in positional],
               **dict((k, getattr(options, k)) for k in kwarg))
        except util.CommandError, e:
            util.err(str(e))


def main():
    parser = OptionParser(
        usage="%prog [options] <commands> [commands options]",
        add_help_option=False)

    commandor_options = [Option('-c', '--config',
                                metavar="FILE", help='Configuration file')]

    manager = Commandor(parser, sys.argv[1:], commandor_options)
    manager.process()

if __name__ == '__main__':
    main()



"""empty message

Revision ID: 34fb6ae55fcb
Revises: None
Create Date: 2012-12-14 01:42:05.349405

"""

# revision identifiers, used by Alembic.
revision = '34fb6ae55fcb'
down_revision = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('users',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(), nullable=True),
    sa.Column('fullname', sa.String(), nullable=True),
    sa.Column('password', sa.String(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    op.create_table('projects',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('name', sa.String(), nullable=True),
    sa.Column('key', sa.String(), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('projects')
    op.drop_table('users')
    ### end Alembic commands ###



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.app
~~~~~~~~~~~~

Gottwall main loop

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""
import importlib

import tornado.ioloop
from jinja2 import Environment, FileSystemLoader
from tornado.web import Application, URLSpec

from handlers import DashboardHandler, LoginHandler, HomeHandler,\
     StatsHandler, MetricsHandler, LogoutHandler, StatsDataSetHandler
from jinja_utils import load_filters, load_globals
from processing import Tasks


## from sqlalchemy import create_engine
## from sqlalchemy.orm import scoped_session, sessionmaker


class HTTPApplication(Application):
    """Base application
    """

    def __init__(self, config):
        self.config = config
        self.db = self.configure_db()
        self.jinja_env = self.configure_env()
        self.data_processor = None
        self.tasks = Tasks()

        params = {"config": self.config,
                  "db": self.db,
                  "env": self.jinja_env}

        self.dirty_handlers = [
            (r"{0}/login".format(self.config['PREFIX']), LoginHandler, params, 'login'),
            (r"{0}/logout".format(self.config['PREFIX']), LogoutHandler, params, 'logout'),
            (r"{0}/dashboard".format(self.config['PREFIX']), DashboardHandler, params, 'dashboard'),
            (r"{0}/(?P<project>.+)/api/stats".format(self.config['PREFIX']),
             StatsHandler, params, 'api-stats'),
            (r"{0}/(?P<project>.+)/api/stats_dataset".format(self.config['PREFIX']),
             StatsDataSetHandler, params, 'api-stats-dataset'),
            (r"{0}/(?P<project>.+)/api/metrics".format(self.config['PREFIX']),
             MetricsHandler, params, 'api-metrics'),
            (r"{0}/".format(self.config['PREFIX']), HomeHandler, params, 'home')]

        config['login_url'] = config['PREFIX'] + config['login_url']

        tornado.web.Application.__init__(
            self, [URLSpec(*x) for x in self.dirty_handlers], **config)

    def configure_env(self):
        """Configure template env
        """
        searchpath = list(self.config.get("TEMPLATES_PATH", 'templates'))

        env = Environment(loader=FileSystemLoader(searchpath),
                          auto_reload=self.config.get('TEMPLATE_DEBUG', False),
                          cache_size=self.config.get('JINJA2_CACHE_SIZE', 50),
                          extensions=self.config.get('JINJA2_EXTENSIONS', ()))
        filters = self.config.get('JINJA2_FILTERS', ())
        globals = self.config.get('JINJA2_GLOBALS', ())

        env.filters.update(load_filters(filters))
        env.globals.update(load_globals(globals))

        return env

    def configure_db(self):
        return None
        # engine = create_engine("{ENGINE}://{USER}:{PASSWORD}@{HOST}:{PORT}/{NAME}".\
        #                             format(**self.config['DATABASE']), echo=False)
        # return scoped_session(sessionmaker(bind=engine))

    def configure_app(self, io_loop=None):
        """Configure application backends and storages
        """
        self.configure_storage(self.config['STORAGE'])

    def configure_storage(self, storage_path):
        """Configure data storage by path

        :param storage: storage path
        """
        module_path, name = storage_path.rsplit('.', 1)
        try:
            module = importlib.import_module(module_path, name)
            storage = getattr(module, name)
        except (ImportError, AttributeError), e:
            raise Exception("Invalid storage: {0}".format(e))
        return storage.setup(self)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.storage.memory
~~~~~~~~~~~~~~~~~~~~~~~

Memory storage for collect statistics

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/GottWall
"""

from itertools import ifilter
from base import BaseStorage
from gottwall.utils import get_by_period, MagicDict, get_datetime
from tornado import gen


class MemoryStorage(BaseStorage):
    """Store keys in dict in memory

    """
    def __init__(self, application):
        super(MemoryStorage, self).__init__(application)
        self._store = MagicDict()
        self._metrics = {}

    def load_db(self, f):
        """Load data from f

        :param f: path to file
        """
        raise RuntimeError

    def save_db(self, f):
        """Save data to file
        """
        raise RuntimeError

    def save_value(self, project, name, period, timestamp, fname=None, fvalue=None, value=1):
        """Save value to store dict

        :paran project: project name
        :param name: metric name
        :param period: period
        :param timestamp:
        :param fname:
        :param fvalue:
        :param value:
        """
        if not isinstance(fvalue, (list, tuple)):
            fvalues = [fvalue]
        else:
            fvalues = fvalue

        for filter_value in fvalues:

            # If cache with key to variable it doesn't work
            if isinstance(self._store[project][name][period][timestamp][fname][filter_value], MagicDict):
                self._store[project][name][period][timestamp][fname][filter_value] = value
            else:
                self._store[project][name][period][timestamp][fname][filter_value] += value
        return True

    @gen.engine
    def save_metric_meta(self, project, name, filters={}, callback=None):
        """Save metric filters

        :param project: project name
        :param name: metric name
        :param filters: metric filters
        """
        if project not in self._metrics:
            self._metrics[project] = {}

        if name not in self._metrics[project].keys():
            self._metrics[project][name] = {}

        if not filters:
            filters = {}

        for f, values in filters.iteritems():

            if f not in self._metrics[project][name].keys():
                self._metrics[project][name][f] = []

            if not isinstance(values, (list, tuple)):
                values = [values]

            for value in values:
                if value not in self._metrics[project][name][f]:
                    self._metrics[project][name][f].append(value)
        if callback:
            callback(True)

    @gen.engine
    def incr(self, project, name, timestamp, value=1, filters=None, callback=None, **kwargs):
        """Add value to metric counter

        :param project: project name
        :param name: metric name
        :param timestamp: timestamp name
        :param value: increment value
        :param filters: dict of filters
        :param \*\*kwargs: additional kwargs
        """

        for period in self._application.config['PERIODS']:
            if filters:
                for fname, fvalue in filters.iteritems():
                    self.save_value(project, name, period,
                                    get_by_period(timestamp, period), fname, fvalue, value)
            self.save_value(project, name, period,
                            get_by_period(timestamp, period), None, None, value)

        self.save_metric_meta(project, name, filters)

        if callback:
            callback(True)

    def decr(self, project, name, timestamp, value=1, filters=None, callback=None, **kwargs):
        return self.incr(project, name, timestamp,  0 - abs(value), filters, **kwargs)

    @gen.engine
    def slice_data(self, project, name, period, from_date=None,
                   to_date=None, filter_name=None, filter_value=None, callback=None):
        """Get statistics by filters
        """

        key = self._store[project][name][period]
        items = [(k, v[filter_name][filter_value]) for k, v in key.items()]
        items.sort(key=lambda x: x[0])

        items = self.filter_by_period(items, period, from_date, to_date)

        if callback:
            callback(items)


    @gen.engine
    def metrics(self, project, callback=None):
        """Return all metrics with filters

        :param project: project name
        :returns: result dict
        """
        if callback:
            if project not in self._metrics.keys():
                callback({})
            else:
                callback(self._metrics[project])



#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall.tests
~~~~~~~~~~~~~~~~

Unittests for gottwall

:copyright: (c) 2011 - 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""
import datetime
import json
import os
import random
from base64 import b64encode

import tornado.gen
from tornado import ioloop

import gottwall.default_config
from base import AsyncBaseTestCase, AsyncHTTPBaseTestCase, RedisTestCaseMixin
from gottwall.aggregator import AggregatorApplication
from gottwall.config import Config
from utils import async_test


HOST = os.environ.get('GOTTWALL_REDIS_HOST', "10.8.9.8")


class TCPBackendTestCase(AsyncBaseTestCase):
    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)

        config.update({"BACKENDS": {"gottwall.backends.tcpip.TCPIPBackend": {}},
                       "STORAGE": "gottwall.storages.MemoryStorage",
                                 "PROJECTS": {"test_project": "secretkey"},
                                 "PRIVATE_KEY": "myprivatekey"})
        self.app = AggregatorApplication(config)
        self.app.configure_app(self.io_loop)

        return self.app


class RedisBackendTestCase(AsyncHTTPBaseTestCase, RedisTestCaseMixin):

    def setUp(self):
        super(RedisBackendTestCase, self).setUp()
        self.client = self._new_client()
        self.client.flushdb()

    def tearDown(self):
        try:
            self.client.connection.disconnect()
            del self.client
        except AttributeError:
            pass
        super(RedisBackendTestCase, self).tearDown()

    def get_new_ioloop(self):
        return ioloop.IOLoop.instance()

    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)

        config.update({"BACKENDS": {"gottwall.backends.redis.RedisBackend": {"HOST": HOST}},
                       "STORAGE": "gottwall.storages.MemoryStorage",
                       "REDIS_HOST": HOST,
                       "PROJECTS": {"test_project": "secretkey2"},
                       "SECRET_KEY": "myprivatekey2"})
        app = AggregatorApplication(config)
        app.configure_app(self.io_loop)
        return app

    @async_test
    @tornado.gen.engine
    def test_subscribe(self):
        app = self.get_app()
        metric_data = {"name": "redis_metric_{0}".format(random.randint(1, 10)),
                       "timestamp": datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S"),
                       "filters": {"views": "anonymouse"},
                       "action": "incr",
                       "value": 2,
                       "type": "bucket"}

        client = self.client
        key = "gottwall:{0}:{1}:{2}".format("test_project",
                                            app.config['PROJECTS']['test_project'],
                                            app.config['SECRET_KEY'])

        (yield tornado.gen.Task(client.publish, key,
                                json.dumps(metric_data)))

        from pprint import pprint
        pprint(app.storage._store)


        self.stop()


class HTTPBackendTestCase(AsyncHTTPBaseTestCase):

    def get_app(self):
        config = Config()
        config.from_module(gottwall.default_config)
        config.update({"BACKENDS": [],
                       "PROJECTS": {"test_project": "secretkey"},
                       "SECRET_KEY": "myprivatekey"})
        self.app = AggregatorApplication(config)
        self.app.configure_app(self.io_loop)
        return self.app

    def test_handler(self):
        app = self.get_app()

        metric_data = {"name": "my_metric_name",
                       "timestamp": datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S"),
                       "filters": {"views": "registered",
                                   "clicks": "anonymouse"},
                       "action": "incr",
                       "value": 2}

        auth_value = "GottWall private_key={0}, public_key={1}".format(
            app.config['SECRET_KEY'],
            app.config['PROJECTS']['test_project'])

        authorization = "{0}:{1}".format(app.config['PROJECTS']['test_project'],
                                         app.config['SECRET_KEY'])

        response = self.fetch("/gottwall/test_project/api/store", method="POST",
                              body=json.dumps(metric_data),
                              headers={"content-type": "application/json",
                                       "Authorization": b64encode(authorization)})

        self.assertEquals(response.body, "OK")
        self.assertEquals(response.code, 200)

        auth_value = "GottWall private_key={0}, public_key={1}".format(
            app.config['SECRET_KEY'],
            app.config['PROJECTS']['test_project'])

        response = self.fetch("/gottwall/test_project/api/store", method="POST",
                              body=json.dumps(metric_data),
                              headers={"content-type": "application/json",
                                       "X-GottWall-Auth": auth_value})

        self.assertEquals(response.body, "OK")
        self.assertEquals(response.code, 200)

        # Test without authorization

        response = self.fetch("/gottwall/test_project/api/store", method="POST",
                              body=json.dumps(metric_data),
                              headers={"content-type": "application/json"})

        self.assertEquals(response.code, 403)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
handlers
~~~~~~~~

module description

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""
import logging
import tornado.escape
from tornado.auth import GoogleMixin
from tornado.web import RequestHandler, HTTPError, asynchronous, authenticated

from jinja2 import TemplateNotFound

from itertools import chain

from tornado.escape import json_decode, json_encode
import tornado.web
import tornado.gen
from tornado import gen

from gottwall import get_version
from gottwall.utils import (timestamp_to_datetime, date_range, get_by_period,
                            date_min, date_max)
from gottwall.settings import DATE_FILTER_FORMAT

logger = logging.getLogger('gottwall')


class User(object):
    """Request user object

    TODO: lookup users in local memory storage
    """
    def __init__(self, handler, request, *args, **kwargs):
        self.username = None
        self.email = None
        self.api_key = None

    def is_authenticated(self):
        return True


class BaseHandler(RequestHandler):
    def __init__(self, *args, **kwargs):
        super(BaseHandler, self).__init__(*args, **kwargs)
        self.set_header("Server", "GottWall/{0}".format(get_version()))

    def initialize(self, config, db, env):
        self.config = config
        self.jinja_env = env
        self.config = config

    def get_current_user(self):
        """ Return current logged user or None.
        """

        user_json = self.get_secure_cookie("user")
        if not user_json:
            return None
        return json_decode(user_json)

    ## @property
    ## def current_user(self):
    ##     """Get request user object
    ##     """

    ##     return User(self, self.request)

    def render(self, template, **kwargs):
        """Render template with \*\*kwargs context

        :param template: template name
        :param \*\*kwargs: template context
        """
        kwargs['handler'] = self
        if 'static' not in kwargs:
            kwargs['static'] = self.config.get('static_url_prefix')

        kwargs['user'] = self.current_user
        kwargs['reverse'] = self.reverse_url
        kwargs['version'] = get_version()
        data = self.render_to_string(template, context=kwargs)
        return self.finish(data)

    def render_to_string(self, template_name, context=None, processors=None):
        context = dict(context or {})
        context['request'] = self.request

        for processor in chain(processors or ()):
            context.update(processor(self.request))

        return self.select_template(template_name).render(context)

    def select_template(self, templates):
        if isinstance(templates, (list, tuple)):
            for template in templates:
                try:
                    return self.jinja_env.get_template(template)
                except TemplateNotFound:
                    continue
        elif isinstance(templates, (str, unicode)):
            return self.jinja_env.get_template(templates)

        raise TemplateNotFound(templates)


class DashboardHandler(BaseHandler):
    @authenticated
    def get(self, *args, **kwargs):

        self.render("dashboard.html", config=self.application.config,
                    projects=self.config['PROJECTS'])


class HomeHandler(BaseHandler):

    def get(self, *args, **kwargs):
        self.redirect(self.reverse_url('dashboard'))


class APIHandler(BaseHandler):
    """Base class for api handlers
    """

    def check_auth(self):
        """Check authorization
        """
        key = self.request.headers.get('Authorization')
        if key != self.application.config['SECRET_KEY']:
            logger.error("Invalid authorixation key: {0}".format(key))
            raise HTTPError(401, "Authorization required")
        return True

    def json_response(self, data, finish=True):
        output_json = tornado.escape.json_encode(data)
        self.set_header("Content-Type", "application/json")
        if finish is True:
            self.finish(output_json)
        else:
            return output_json


class StatsMixin(object):

    def convert_date_range(self, from_date, to_date):
        """Convert str from_date and to_data objects to
        datetime object

        :param from_date: from date string
        :param to_date: to date string
        :return: tuple (from_date, to_date)
        """

        from_date = timestamp_to_datetime(from_date, DATE_FILTER_FORMAT) if from_date else from_date
        to_date = timestamp_to_datetime(to_date, DATE_FILTER_FORMAT) if to_date else to_date
        return from_date, to_date

    def get_params(self):
        name = self.get_argument('name', None)
        from_date = self.get_argument('from_date', None)
        to_date = self.get_argument('to_date', None)
        period = self.get_argument('period', 'week')
        filter_name = self.get_argument('filter_name', None)
        filter_value = self.get_argument('filter_value', None)

        return name, from_date, to_date, period, filter_name, filter_value

    def clean_period(self, from_date, to_date):
        try:
            from_date, to_date = self.convert_date_range(from_date, to_date)
        except ValueError:
            self.set_status(400)
            self.json_response({"text": "Invalid date range params"})
            return None, None
        return from_date, to_date


    def validate_name(self, name, period):
        if not all([name, period]):
            self.set_status(400)
            self.json_response({"text": "You need specify name and period"})
            return
        return True


class StatsHandler(APIHandler, StatsMixin):
    """Load periods statistics
    """

    @authenticated
    @tornado.web.asynchronous
    @gen.engine
    def get(self, project, *args, **kwargs):

        try:
            name, from_date, to_date, period, filter_name, filter_value = self.get_params()
        except Exception:
            self.set_status(400)
            self.json_response({"text": "Bad request"})
            return

        from_date, to_date = self.clean_period(from_date, to_date)

        if self.validate_name(name, period) and from_date and to_date:

            data = yield gen.Task(self.application.storage.slice_data,
                                  project, name, period, from_date, to_date, filter_name, filter_value)

            self.json_response({"range": list(data),
                                "project": project,
                                "period": period,
                                "name": name,
                                "filter_name": filter_name,
                                "filter_value": filter_value,
                                "avg": 0})


class StatsDataSetHandler(APIHandler, StatsMixin):
    """Load data for filters without filter value
    """
    @authenticated
    @tornado.web.asynchronous
    @gen.engine
    def get(self, project, *args, **kwargs):

        try:
            name, from_date, to_date, period, filter_name, filter_value = self.get_params()
            from_date, to_date = self.convert_date_range(from_date, to_date)

        except Exception:
            self.set_status(400)
            self.json_response({"text": "Bad request"})
            return

        from_date, to_date = self.clean_period(from_date, to_date)

        if self.validate_name(name, period) and from_date and to_date:

            data = yield gen.Task(self.application.storage.slice_data_set,
                                  project, name, period, from_date, to_date, filter_name)

            self.json_response({"data": data,
                                "project": project,
                                "period": period,
                                "name": name,
                                "filter_name": filter_name,
                                "date_range": [get_by_period(x, period)
                                               for x in date_range(date_min(from_date, period),
                                                                   date_max(to_date, period), period)]})


class MetricsHandler(APIHandler):
    """Load metrics structure
    """
    @authenticated
    @tornado.web.asynchronous
    @gen.engine
    def get(self, project, *args, **kwargs):
        metrics = yield gen.Task(self.application.storage.metrics, project)
        self.json_response(metrics)


class LogoutHandler(BaseHandler):

    def get(self):
        self.clear_cookie('user')
        self.redirect("/")


class LoginHandler(BaseHandler, GoogleMixin):

    @asynchronous
    def get(self):
        if self.get_argument("openid.mode", None):
            self.get_authenticated_user(self.async_callback(self._on_auth))
            return
        self.authenticate_redirect()

    def _on_auth(self, user):

        if not user:
            raise HTTPError(500, "Google auth failed")

        if self.application.config.get('USERS') and \
               user.get('email').lower() not in [x.lower() for x in self.application.config['USERS']]:
            raise HTTPError(403, "%s access forbiden." % user.get('name'))

        self.set_secure_cookie("user", json_encode(user))
        self.redirect("/")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall
~~~~~~~~

Simple statistics aggregator


:copyright: (c) 2012 - 2013 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

__all__ = 'get_version',
__author__ = "GottWal team"
__license__ = "BSD, see LICENSE for more details"
__maintainer__ = "Alexandr Lispython"

try:
    __version__ = __import__('pkg_resources') \
        .get_distribution('human_curl').version
except Exception, e:
    __version__ = 'unknown'

if __version__ == 'unknown':
    __version_info__ = (0, 0, 0)
else:
    __version_info__ = __version__.split('.')
__build__ = 0x000034


def get_version():
    return __version__



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.jinja_utils
~~~~~~~~~~~~~~~~~~~~

Jinja configuration utils

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/GottWall
"""


def load_filters(filters):
    result = {}
    for m in filters:
        f = __import__(m, fromlist=['filters'])
        result.update(f.filters)
    return result

def load_extensions(extensions):
    """
    Load the extensions from the list and bind it to the environment.
    Returns a dict of instanciated environments.
    """
    result = {}
    for extension in extensions:
        if isinstance(extension, basestring):
            extension = import_string(extension)
        result[extension.identifier] = extension(environment)
    return result

def load_globals(globals, key="globals"):
    result = {}
    for m in globals:
        f = __import__(m, fromlist=[key])
        result.update(f.globals)
    return result



#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall.tests
~~~~~~~~~~~~~~~~

Unittests for gottwall

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

import os
import unittest
from tornado.testing import AsyncHTTPTestCase, AsyncTestCase

class SettingsMixin(object):

    @property
    def redis_settings(self):
        env = os.environ

        return {"HOST": env.get('GOTTWALL_REDIS_HOST', "10.8.9.8"),
                "PORT": env.get('GOTTWALL_REDIS_PORT', 6379),
                "DB": env.get('GOTTWALL_REDIS_DB', 0)}

class BaseTestCase(unittest.TestCase, SettingsMixin):
    pass

class AsyncBaseTestCase(AsyncTestCase, SettingsMixin):
    pass

class AsyncHTTPBaseTestCase(AsyncHTTPTestCase, SettingsMixin):
    pass


import tornadoredis

class TestRedisClient(tornadoredis.Client):

    def __init__(self, *args, **kwargs):
        self._on_destroy = kwargs.get('on_destroy', None)
        if 'on_destroy' in kwargs:
            del kwargs['on_destroy']
        super(TestRedisClient, self).__init__(*args, **kwargs)

    def __del__(self):
        super(TestRedisClient, self).__del__()
        if self._on_destroy:
            self._on_destroy()



class RedisTestCaseMixin(object):

    def setUp(self):
        super(RedisTestCaseMixin, self).setUp()
        self.client = self._new_client()
        self.client.flushdb()

    def tearDown(self):
        try:
            self.client.connection.disconnect()
            del self.client
        except AttributeError:
            pass
        super(RedisTestCaseMixin, self).tearDown()


    def _new_client(self, pool=None, on_destroy=None):
        client = TestRedisClient(io_loop=self.io_loop,
                                 host=self.redis_settings['HOST'],
                                 port=self.redis_settings['PORT'],
                                 selected_db=self.redis_settings['DB'],
                                 connection_pool=pool,
                                 on_destroy=on_destroy)

        return client



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.aggregator
~~~~~~~~~~~~~~~~~~~

Gottwall main loop

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/gottwall/gottwall
"""
import importlib
import time
from logging import getLogger

import tornado.ioloop
from tornado.web import Application, URLSpec

from backends import HTTPBackend as HTTPBackendHandler
from processing import PeriodicProcessor, Tasks


## from sqlalchemy import create_engine
## from sqlalchemy.orm import scoped_session, sessionmaker

logger = getLogger()


class AggregatorApplication(Application):
    """Base application
    """

    def __init__(self, config):
        self.config = config
        self.data_processor = None
        self.tasks = Tasks()
        self.backends = []

        params = {"config": self.config,
                  "app": self}

        self.dirty_handlers = [
            # Default HTTP backend
            (r"{0}/(?P<project>.+)/api/store".format(self.config['PREFIX']),
             HTTPBackendHandler, params, 'store')]

        tornado.web.Application.__init__(
            self, [URLSpec(*x) for x in self.dirty_handlers], **config)

    def configure_app(self, io_loop=None):
        """Configure application backends and storages
        """
        storage = self.configure_storage(self.config['STORAGE'])
        self.configure_backends(self.config['BACKENDS'], io_loop, self.config,
                                storage, self.tasks)

        # Add periodic processing
        self.data_processor = PeriodicProcessor(self, io_loop=io_loop)
        self.data_processor.start()

    def configure_storage(self, storage_path):
        """Configure data storage by path

        :param storage: storage path
        """
        module_path, name = storage_path.rsplit('.', 1)
        try:
            module = importlib.import_module(module_path, name)
            storage = getattr(module, name)
        except (ImportError, AttributeError), e:
            raise Exception("Invalid storage: {0}".format(e))
        return storage.setup(self)

    def configure_backends(self, backends, io_loop, config, storage, tasks):
        """Configture data receive backends

        :param backends: list of backends
        """
        for backend_path in backends:
            module_path, name = backend_path.rsplit('.', 1)
            try:
                backend_module = importlib.import_module(module_path, name)

                backend = getattr(backend_module, name)
            except (ImportError, AttributeError), e:
                raise Exception("Invalid backend: {0}".format(e))
            self.backends.append(backend.setup_backend(self, io_loop, config, storage, tasks))

        return True

    def shutdown(self):
        """Shutdown application
        """
        for backend in self.backends:
            backend.shutdown()

    def check_ready_to_stop(self, callback=None):
        """Check that all backends flush data
        """
        io_loop = tornado.ioloop.IOLoop.instance()

        if all([backend.ready_to_stop() for backend in self.backends]):
            logger.info("All backends ready to stop")
            io_loop.add_timeout(time.time() + 2, io_loop.stop)
            return True

        logger.info("Not all backend ready to stop")
        for backend in self.backends:
            logger.info("Backend {0} has {1} tasks in progress".format(repr(backend), backend.current_in_progress))

        io_loop.add_timeout(time.time() + 2, self.check_ready_to_stop)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.tcpip
~~~~~~~~~~~~~~

Raw TCP/IP backend for gottwall messages

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details..
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""

from tornado.netutil import TCPServer
from tornado.util import b

from gottwall.backends.base import BaseBackend


class TCPIPBackend(TCPServer, BaseBackend):

    def __init__(self, application, io_loop, config, storage, tasks, *args, **kwargs):
        self.io_loop = io_loop
        self.config = config
        self.storage = storage
        self.tasks = tasks
        self.application = application
        self.working = True
        self.current_in_progress = 0
        super(TCPIPBackend, self).__init__(*args, **kwargs)

    @classmethod
    def setup_backend(cls, application, io_loop, config, storage, tasks):
        """Install backend to ioloop

        :param ioloop: :class:`tornadoweb.ioloop.IOLoop` instance
        :param config: :class:`~gottwall.config.Config` instance
        """

        server = cls(application, io_loop, config, storage, tasks)
        server.listen(server.backend_settings.get('PORT', 8897))
        return server

    def handle_stream(self, stream, address):
        """Process stream data

        :param  stream: :class:`tornado.iostream` instance
        :param address: client address
        """
        stream.read_until(b("\r\n\r\n"), self.callback)
        stream.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.models
~~~~~~~~~~~~~~~

Model that store statistics info

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""
from sqlalchemy import Column, Integer, String, Sequence
from sqlalchemy.ext.declarative import declarative_base


Base = declarative_base()

class Project(Base):
    """Metric project
    """
    __tablename__ = "projects"
    id = Column(Integer, Sequence('project_id_seq'), primary_key=True)
    name = Column(String)
    key = Column(String) # Private api key


    def __repr__(self):
        return u"<Project({0}, {1}".format(self.id, self.name)


class User(Base):
    __tablename__ = 'users'

    id = Column(Integer, Sequence('user_id_seq'), primary_key=True)
    name = Column(String)
    fullname = Column(String)
    password = Column(String)

    def __repr__(self):
        return u"<User('%s','%s', '%s')>" % (self.name, self.fullname, self.password)



#!/usr/bin/env python
# -*- coding:  utf-8 -*-
"""
gottwall
~~~~~~~~

Realtime statistics aggregation platform

:copyright: (c) 2012 - 2013 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""

import sys
import os
try:
    import subprocess
    has_subprocess = True
except:
    has_subprocess = False

from setuptools import setup, find_packages

try:
    readme_content = open(os.path.join(os.path.abspath(
        os.path.dirname(__file__)), "README.rst")).read()
except Exception, e:
    print(e)
    readme_content = __doc__


VERSION = "0.2.10"


def run_tests():
    from tests import suite
    return suite()

py_ver = sys.version_info

#: Python 2.x?
is_py2 = (py_ver[0] == 2)

#: Python 3.x?
is_py3 = (py_ver[0] == 3)

tests_require = [
    'nose',
    'mock==1.0.1']

install_requires = [
    "redis==2.7.2",
    "tornado==2.4.1",
    "python-dateutil==2.1",
    "tornado-redis==2.4.1",
    "commandor==0.1.5",
    "SQLAlchemy==0.7.9",
    "alembic==0.4.0",
    "Jinja2==2.6"]

if not (is_py3 or (is_py2 and py_ver[1] >= 7)):
    install_requires.append("importlib==1.0.2")

PACKAGE_DATA = []
PROJECT = 'gottwall'
for folder in ['static', 'templates']:
    for root, dirs, files in os.walk(os.path.join(PROJECT, folder)):
        for filename in files:
            PACKAGE_DATA.append("%s/%s" % (root[len(PROJECT) + 1:], filename))

setup(
    name="gottwall",
    version=VERSION,
    description="Realtime statistics aggregation platform",
    long_description=readme_content,
    author="Alex Lispython",
    author_email="alex@obout.ru",
    maintainer="Alexandr Lispython",
    maintainer_email="alex@obout.ru",
    url="https://github.com/gottwall/gottwall",
    packages=find_packages(),
    package_data={'': PACKAGE_DATA},
    entry_points={
        'console_scripts': [
            'gottwall = gottwall.runner:main',
        ]},
    install_requires=install_requires,
    tests_require=tests_require,
    license="BSD",
#    test_suite="nose.collector",
    platforms = ['Linux', 'Mac'],
    classifiers=[
        "Environment :: Web Environment",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries",
        "Development Status :: 4 - Beta",
        "Topic :: System :: Installation/Setup",
        "Topic :: System :: Logging",
        "Topic :: System :: Monitoring",
        "Topic :: System :: Networking"
        ],
    test_suite = '__main__.run_tests'
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.backends.base
~~~~~~~~~~~~~~~~~~~~~~

Base backends for metric calculation

:copyright: (c) 2012 - 2013 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/gottwall/gottwall
"""
import json
from base64 import b64decode

import tornado.gen
from tornado.web import HTTPError

from gottwall.backends.base import BaseBackend
from gottwall.handlers import BaseHandler
from gottwall.utils import parse_dict_header


class HTTPBackend(BaseBackend, BaseHandler):

    def initialize(self, config, app):
        self.config = config
        self.application = app
        self.current_in_progress = 0

    ## def __init__(self, io_loop, config, storage, *args, **kwargs):
    ##     self.io_loop = io_loop
    ##     self.config = config
    ##     self.storage = storage
    ##     super(HTTPBackend, self).__init__(*args, **kwargs)

    @staticmethod
    def merge_handlers(app):
        """Merge hosts handlers in application handler
        """
        hosts = {}
        handlers = app.handlers
        for host, patterns in handlers:
            if host not in hosts.keys():
                hosts[host] = []
            hosts[host] += patterns
        app.handlers = [(host, patterns) for host, patterns in hosts.items()]

    @tornado.gen.engine
    def post(self, project, *args, **kwargs):
        self.storage = self.application.storage

        if not self.validate_project(project):
            raise HTTPError(404, "Invalid project")

        if not self.validate_content_type():
            raise HTTPError(400, "Invalid content type")

        if not self.check_auth(project):
            raise HTTPError(403, "Forbidden")

        data = json.loads(self.request.body)
        self.process_data(project, data)

        self.write("OK")
        self.finish()

    def validate_project(self, project):
        """Validate projects name
        """
        return True

    def validate_content_type(self):
        if self.request.headers['content-type'] == 'application/json':
            return True
        return False

    def check_auth(self, project):
        """Check authorization headers
        """

        header = self.request.headers.get("Authorization", None)
        gottwall_header = self.request.headers.get("X-GottWall-Auth", None)

        if gottwall_header:
            return self.check_gottwall_auth(gottwall_header, project)

        if not header:
            return False

        if header.startswith('Basic '):
            header = header[6:].strip()

        return self.check_basic_auth(header, project)

    def check_basic_auth(self, header, project):
        """Parse basic authorization header

        :param header: authorization header value
        """
        auth_info = header.split(None, 1)

        public_key, private_key = b64decode(auth_info[0]).split(":")

        return self.check_key(private_key, public_key, project)

    def check_gottwall_auth(self, header, project):
        """Check X-GottWall-Auth

        :param header: header string value
        :param project: project name
        """
        if header.startswith('GottWall'):
            header = header[8:]

        params = parse_dict_header(header)
        return self.check_key(params.get('private_key'),
                              params.get('public_key'), project)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.storages.redis
~~~~~~~~~~~~~~~~~~~~~~~

Redis storage for collect statistics

:copyright: (c) 2012 - 2013 by GottWall Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/gottwall/gottwall
"""
from logging import getLogger

import tornado.gen
import tornadoredis
from tornado import gen
from tornado.gen import Task
from tornado.escape import to_unicode
from tornadoredis.exceptions import ConnectionError

from gottwall.settings import STORAGE_SETTINGS_KEY
from gottwall.storages.base import BaseStorage
from gottwall.utils import get_datetime, get_by_period


logger = getLogger()


class Client(tornadoredis.Client):
    def __init__(self, reconnect_callback=None, **kwargs):
        self._reconnect_callback = reconnect_callback
        super(Client, self).__init__(**kwargs)

    ## def on_disconnect(self):
    ##     if self.subscribed:
    ##         self.subscribed = False

    ##     # self._reconnect_callback()
    ##     raise ConnectionError("Connection lost")


class RedisStorage(BaseStorage):
    """Redis backend to store statistics

    Store counters in redis hash keys
    Key format:
    project_name;mertic_name;period;filter_name1|value1/filter_name2|value2 {
        "2012": 100,
        "2013": 200
        }
    """

    def __init__(self, application):
        super(RedisStorage, self).__init__(application)
        config = self._application.config

        self.client = Client(
            host=config[STORAGE_SETTINGS_KEY].get('HOST', 'localhost'),
            port=int(config[STORAGE_SETTINGS_KEY].get('PORT', 6379)),
            password=config[STORAGE_SETTINGS_KEY].get('PASSWORD', None),
            selected_db=int(config[STORAGE_SETTINGS_KEY].get('DB', 0)))

        logger.info("Redis storage client {0}".format(repr(self.client)))
        self.client._reconnect_callback = self.client.connect

        self.client.connect()
        self.client.select(self.client.selected_db)

    @gen.engine
    def save_metric_meta(self, pipe, project, name,
                         filters=None, callback=None):
        """Save metric filters

        :param project: project name
        :param name: metric name
        :param filters: metric filters
        """
        if not filters:
            filters = {}

        pipe.sadd(self.get_metrics_key(project), name)

        for f, values in filters.iteritems():
            pipe.sadd(self.get_filters_names_key(project, name), f)

            if isinstance(values, (list, tuple)):
                for value in values:
                    pipe.sadd(self.get_filters_values_key(project, name, f), value)
            else:
                pipe.sadd(self.get_filters_values_key(project, name, f), values)

        if callback:
            callback()

    def get_metrics_key(self, project):
        """Get key for metrics list

        :param project: project name
        """
        return "{0}-metrics".format(project)

    def get_filters_values_key(self, project, metric_name, f):
        """Get key for filter values list

        :param project:
        :param metric_name:
        :param f: filter name
        """
        return u"{0}-metrics-filter-values:{1}:{2}".format(project,
                                                           to_unicode(metric_name),
                                                           to_unicode(f))

    def get_filters_names_key(self, project, metric_name):
        """Get key for metrics filters names

        :param project: project name
        :param metric_name: metric name
        """
        return u"{0}-metrics-filters:{1}".format(project, to_unicode(metric_name))

    @tornado.gen.engine
    def incr(self, project, name, timestamp, value=1, filters=None, callback=None, **kwargs):
        """Make incr in redis hash
        """

        pipe = self.client.pipeline(transactional=True)
        pipe.select(self.client.selected_db)

        for period in self._application.config['PERIODS']:
            if filters:
                for fname, fvalue in filters.iteritems():
                    if not isinstance(fvalue, (list, tuple)):
                        # Make incr for all values
                        fvalue = [fvalue]

                    for fvalue_item in fvalue:
                        pipe.hincrby(self.make_key(
                            project, name, period, {fname: fvalue_item}),
                                     get_by_period(timestamp, period), value)

            pipe.hincrby(self.make_key(project, name, period), get_by_period(timestamp, period), value)

            self.save_metric_meta(pipe, project, name, filters)

        res = (yield gen.Task(pipe.execute))

        if callback:
            callback(res)

    def make_key(self, project, name, period, filters=None):
        """Make key from parameters

        :param project: project name
        :param name: mertric name
        :param period: period prefix
        :param timestamp: timestamp
        :param fname: filter name
        :param fvalue: fil

        """

        parts = [project, name, period]

        if isinstance(filters, dict):
            filters_part = u"/".join(
                [u"{0}|{1}".format(f, to_unicode(filters[f]))
                 for f in sorted(filters.keys(), key=lambda x: x) if f])

            if filters_part:
                parts.append(filters_part)

        return u';'.join(parts)


    @gen.engine
    def slice_data(self, project, name, period, from_date=None, to_date=None,
                   filter_name=None, filter_value=None, callback=None):


        key = self.make_key(project, name, period,
                            {filter_name: filter_value})

        items = yield Task(self.client.hgetall, key)

        if callback:
            callback(self.filter_by_period(map(lambda x: (get_datetime(x[0], period), int(x[1])), items.iteritems()),
                                           period, from_date, to_date))

    @gen.engine
    def slice_data_set(self, project, name, period, from_date=None, to_date=None,
                       filter_name=None, callback=None):

        client = self.client

        filter_values = sorted((yield gen.Task(client.smembers,
                                               self.get_filters_values_key(project, name, filter_name))))
        items = {}

        for value in filter_values:
            items[value] = {}
            tmp_range = yield Task(self.client.hgetall,
                                   self.make_key(project, name, period, {filter_name: value}))

            items[value]['range'] = self.filter_by_period(map(lambda x: (get_datetime(x[0], period),
                                                                         int(x[1])), tmp_range.items()),
                                                          period, from_date, to_date)

            filtered_range_values = map(lambda x: int(x[1]), items[value]['range'])
            items[value]['avg'] = (sum(filtered_range_values) / len(items[value]['range'])) \
                                  if (len(items[value]['range']) > 0) else 0
            items[value]['max'] = max(filtered_range_values)
            items[value]['min'] = min(filtered_range_values)

        if callback:
            callback(items)

    @tornado.gen.engine
    def metrics(self, project, callback=None):
        """Return all metrics with filters

        :param project: project name
        :returns: result dict
        """
        client = self.client

        self.client.select(self.client.selected_db)

        metrics = {}

        for metric_name in (yield gen.Task(client.smembers,
                                       self.get_metrics_key(project))):
            if metric_name not in metrics.keys():
                metrics[metric_name] = {}

            for filter_name in (yield gen.Task(client.smembers,
                                           self.get_filters_names_key(project, metric_name))):
                metrics[metric_name][filter_name] = list(
                    sorted((yield gen.Task(client.smembers,
                                       self.get_filters_values_key(project, metric_name, filter_name)))))

        if callback:
            callback(metrics)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
gottwall.utils
~~~~~~~~~~~~~~

Core GottWall utilities

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: , see LICENSE for more details.
:github: http://github.com/gottwall/gottwall
"""
import os.path
from datetime import datetime, timedelta, date
from urllib2 import parse_http_list
from dateutil.relativedelta import relativedelta
from dateutil.rrule import rrule, MINUTELY, MONTHLY, DAILY, HOURLY


# Backport of OrderedDict() class that runs on Python 2.4, 2.5, 2.6, 2.7 and pypy.
# Passes Python2.7's test suite and incorporates all the latest updates.

try:
    from thread import get_ident as _get_ident
except ImportError:
    from dummy_thread import get_ident as _get_ident

try:
    from _abcoll import KeysView, ValuesView, ItemsView
except ImportError:
    pass


from settings import PROJECT_ROOT, TIMESTAMP_FORMAT, PERIOD_PATTERNS


__all__ = 'rel',


def rel(*args):
    return os.path.join(PROJECT_ROOT, *args)


def timestamp_to_datetime(timestamp, format=TIMESTAMP_FORMAT):
    """Convert `timestamp` to `datetime`

    :param timestamp: str
    """
    if isinstance(timestamp, (str, unicode)):
        return datetime.strptime(timestamp, format)
    return timestamp


def get_by_period(dt, period):
    """Ge"%Y-%m-%dT%H:%M"t period value by timestamp

    :param dt: `datetime.datetime` instance
    :param period: period name
    :returns: str repr of timestamp
    """
    ts = timestamp_to_datetime(dt)

    if period == "week":
        return "{0}-{1}".format(ts.year, ts.isocalendar()[1])
    elif period in PERIOD_PATTERNS:
        return ts.strftime(PERIOD_PATTERNS[period])
    return None


def get_datetime(timestamp, period):
    """Convert str timestamp to datetime object

    Reverse for ``get_by_period``

    :param timestamp: str representation of time
    :param perid: period
    """
    if period == 'week' and isinstance(timestamp, (str, unicode)):
        year, week = timestamp.split("-")
        ret = datetime.strptime("{0}-{1}-1".format(year, week), '%Y-%W-%w')
        if date(int(year), 1, 4).isoweekday() > 4:
            ret -= timedelta(days=7)
        return ret
    if isinstance(timestamp, datetime):
        return timestamp
    elif period in PERIOD_PATTERNS:
        return datetime.strptime(timestamp, PERIOD_PATTERNS[period])
    return None


def parse_dict_header(value):
    """Parse key=value pairs from value list
    :param value: header string
    :return: params dict
    """
    result = {}
    for item in parse_http_list(value):
        if "=" not in item:
            result[item] = None
            continue
        name, value = item.split('=', 1)
        if value[:1] == value[-1:] == '"':
            value = value[1:-1] # strip "
        result[name] = value
    return result


class MagicDict(dict):
    """Dict that create key with value dict if
    if does not exist
    """

    def __getitem__(self, key):
        if key not in self:
            self[key] = self.__class__()
        return dict.__getitem__(self, key)


def date_range(from_date, to_date, period="month"):

    rrule_period = None

    if period == "month":
        rrule_period = MONTHLY
    elif period == "hour":
        rrule_period = HOURLY
    elif period == "minute":
        rrule_period = MINUTELY
    elif period == "day":
        rrule_period = DAILY
    else:
        return []

    return list(rrule(rrule_period, dtstart=from_date, until=to_date))

def date_min(from_date, period):
    if period == "year":
        return from_date.replace(month=1, hour=0, day=1, minute=0, second=0)
    elif period == "month":
        return from_date.replace(hour=0, day=1, minute=0, second=0)
    return from_date

def date_max(to_date, period):
    if period == "year":
        return to_date.replace(month=12, hour=23, day=31, minute=59, second=59)
    elif period == "month":
        to_date = to_date.replace(day=1, hour=23, minute=59, second=59) + relativedelta(months=+1, days=-1)
    return to_date



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
gottwall.backends.redis
~~~~~~~~~~~~~~~~~~~~~~~

Redis pub/sub backend

:copyright: (c) 2012 by GottWall team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/gottwall
"""
from logging import getLogger

import tornado.ioloop
import tornadoredis
from tornado import gen
from tornadoredis.exceptions import ConnectionError

from gottwall.backends.base import BaseBackend
from gottwall.processing import RedisBackendPeriodicProcessor


logger = getLogger()


class Client(tornadoredis.Client):
    def __init__(self, reconnect_callback=None, **kwargs):
        self._reconnect_callback = reconnect_callback
        super(Client, self).__init__(**kwargs)


class RedisBackend(BaseBackend):

    def __init__(self, application, io_loop, config, storage, tasks, data_threshold=500):
        self.io_loop = io_loop
        self.config = config
        self.storage = storage
        self.client = None
        self.tasks = tasks
        self.data_processing_threshold = data_threshold
        self.current_in_progress = 0
        self.data_processor = None
        self.application = application
        self.working = True

    def get_redis_client(self):
        """Get redis client instance
        """

        client = Client(
            #connection_pool=connection_pool,
            host=self.backend_settings.get('HOST', 'localhost'),
            port=int(self.backend_settings.get('PORT', 6379)),
            io_loop=self.io_loop,
            password=self.backend_settings.get('PASSWORD', None),
            selected_db=int(self.backend_settings.get('DB', 0)),
            reconnect_callback=self.listen)

        return client

    def configure_client(self):
        """Make redis client instance
        """
        self.client = self.get_redis_client()
        return self.client

    def configure_processors(self):
        self.data_processor = RedisBackendPeriodicProcessor(self,
            io_loop=self.io_loop, config=self.config)
        self.data_processor.start()

    @classmethod
    def setup_backend(cls, application, io_loop, config, storage, tasks):
        """Setup data handler

        :param io_loo: :class:`tornado.ioloop.IOLoop` object
        :param confi: :class:`gottwall.config.Config` object
        :param storage: :class:`gottwall.storage.Storage` object
        """
        backend = cls(application, io_loop, config, storage, tasks)
        backend.configure_client()
        backend.configure_processors()
        backend.listen()
        return backend

    @gen.engine
    def listen(self):
        """Listen socket

        :param client: redis client
        """
        self.client.connect()

        yield tornado.gen.Task(
            self.client.psubscribe,
            "{0}:*".format(self.backend_settings.get('CHANNEL', 'gottwall')))
        self.client.listen(self.callback)

    def callback(self, message):

        if message.body == 1:
            # handshake
            return

        try:
            project, public_key, private_key = self.parse_channel(message.channel)
        except ValueError:
            logger.warn("Invalid channel credentails")
            return

        data = self.parse_data(message.body.strip())
        message_type = data.get('type', None)

        if message_type == 'notification':
            # Load data from key to deque for project
            self.load_buckets(project)
        elif message_type == 'bucket':
            self.process_data(project, self.parse_data(data))

        return True

    def parse_channel(self, channel):
        """Parser private, public keys from channel name

        :param message: message object
        :return: tuple for (project, public, private)
        """
        return channel.split(":")[1:]

    def bucket_key(self, project):
        """Build bucket key

        :param project: project name
        """
        return "{0}:{1}:{2}".format(
            self.backend_settings.get('CHANNEL', 'gottwall'),
            project, self.config['PROJECTS'][project])

    @gen.engine
    def load_buckets(self, project):
        """Load data from key to application deque

        :param project: project name
        """
        client = self.get_redis_client()
        key = self.bucket_key(project)
        length = (yield gen.Task(client.scard, key))

        # Max load elements at once
        i = min(int(self.backend_settings.get("MAX_LOADING", 100)), length)

        while i > 0 and (self.current_in_progress < self.data_processing_threshold) and self.working:
            raw_data = (yield gen.Task(client.spop, key))

            if not raw_data:
                break

            self.current_in_progress += 1
            try:
                self.process_data(project, self.parse_data(raw_data),
                                  self.process_data_callback)
            except Exception, e:
                logger.warning(e)
                print(e)

            i -= 1

    def process_data_callback(self, res):
        """Function that called then data processes

        :param res: process result
        """
        self.current_in_progress -= 1

    def shutdown(self):
        """Shutdown backend

        """
        self.working = False
        logger.debug("Tasks in progress {0}".format(self.current_in_progress))
